# Pantera Passive Plugin - Check if page set cookies

# FILENAME      : check_for_cookies.py
# CODER         : Simon Roses Femerling
# DATE          : 07/20/2006
# LAST UPDATE   : 
# ABSTRACT      : Check if page set cookies
#       
# - Roses Labs Innovations (RL+I)
# Roses Labs
# http://www.roseslabs.com
#
# Copyright (c) 2003-2006 Roses Labs.
#
# You may not distribute, transmit, repost this software for commercial 
# purposes without Roses Labs written permission. 
#
# Permission is hereby granted, free of charge, to any person obtaining
# a copy of this software and associated documentation files (the
# "Software"), to deal in the Software without restriction, including
# without limitation the rights to use, copy, modify, publish,
# distribute the Software, and to permit persons to whom the Software 
# is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
# IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
# CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
# TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
# SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
#

'''
@author:       Simon Roses Femerling
@license:      GNU General Public License 2.0 or later
@contact:      pantera.proxy@gmail.com
@organization: OWASP / Roses Labs
'''

from panteraPlugins import PPM

plugin_info =  {'id':"CheckForCookies",
                'name':"Set-Cookies Check",
                'author':"Simon Roses Femerling",
                'email':"sroses@roseslabs.com",
                'version':"0.1",
                'desc':"Check if page sets cookies",
                'tag':"cookie"  # defined tags else ignore plugin
                } 

class CheckForCookies(PPM):
    '''
    CheckForCookies class
    '''
    
    def InitAnalyzer(self):
        '''
        Init function.
        '''
        PPM.__init__(self)

    def BeginAnalyzer(self,obj):
        '''
        Begin function.
        '''
        self.SetObjData(obj)
        for h in obj.serverheader.headerValuesDict.keys():
            t = h.lower()
            if t == 'set-cookie' or t == 'set-cookie2':
                co = obj.serverheader.headerValuesDict[h]
                self.SetType('cookie')
                for c in co:
                    self.SetTypeData(c)
                self.SetLevel('info')
                self.SetResult(1,'cookie')
        
# RL+I EOF